@@ -47,6 +47,9 @@
 #define T_RETRY	    (1*SEC)	/* 1 sec */
 #define R_RETRY	    25000	/* enough try keep trying during a long
 				   weekend */
+#define T_REGISTER  (5*SEC)	/* 5 sec, registration with ATMARP server */
+#define R_REGISTER  0		/* no explicit retries */
+
 #define MAX_DELAY   (300*SEC)	/* never wait more than 5 min for anything */
 
 
@@ -130,6 +133,9 @@ static int want_arp_srv(const ITF *itf)
 }
 
 
+static void timer_expiration(void *user);
+
+
 void vcc_detach(ENTRY *entry)
 {
     ENTRY *walk;
@@ -137,11 +143,13 @@ void vcc_detach(ENTRY *entry)
     /* immediately try to bring it up again if this was the connection to the
        ATMARP server and we still need it. Should delay this in case the ARP
        server has a real problem. @@@ */
+    /* UPDATE: the delay might work now ... */
     if (entry->itf && entry->itf->arp_srv == entry)
 	for (walk = entry->itf->table; walk; walk = walk->next) {
 	    if (walk->state == as_resolv) break;
 	if (walk) {
-	    (void) want_arp_srv(entry->itf);
+	    if (entry->state == as_valid) START_TIMER(entry,REGISTER);
+	    /* (void) want_arp_srv(entry->itf); */
 	    return;
 	}
     }
@@ -280,10 +288,6 @@ static void arp_reply(VCC *vcc,uint32_t src_ip,
   struct sockaddr_atmsvc *src_addr,uint32_t tgt_ip,
   struct sockaddr_atmsvc *tgt_addr)
 {
-    if (!src_addr) {
-	arp_nak(vcc,src_ip,tgt_ip,tgt_addr);
-	return;
-    }
     diag(COMPONENT,DIAG_DEBUG,"sending ARP reply");
     send_arp(vcc->fd,ARPOP_REPLY,src_ip,src_addr,tgt_ip,tgt_addr);
 }
@@ -315,9 +319,6 @@ static void send_notifications(ENTRY *entry,int success)
 }
 
 
-static void timer_expiration(void *user);
-
-
 static void revalidate(ENTRY *entry)
 {
     entry->state = as_resolv;
@@ -356,7 +357,7 @@ static void timer(ENTRY *entry)
 
 static void timeout(ENTRY *entry)
 {
-    VCC *vcc;
+    VCC *vcc,*next;
 
     entry->timer = NULL;
     switch (entry->state) {
@@ -376,11 +377,15 @@ static void timeout(ENTRY *entry)
 		discard_entry(entry);
 		break;
 	    }
-	    for (vcc = entry->vccs; vcc; vcc = vcc->next)
+	    for (vcc = entry->vccs; vcc; vcc = next) {
+		next = vcc->next;
 		if (!vcc->connecting)
-		    if (set_ip(vcc->fd,0) < 0)
+		    if (set_ip(vcc->fd,0) < 0) {
 			diag(COMPONENT,DIAG_ERROR,"set_ip(0): %s",
 			  strerror(errno));
+			disconnect_vcc(vcc);
+		    }
+	    }
 	    if (entry->svc && entry->itf->arp_srv &&
 	      !(entry->flags & ATF_ARPSRV)) revalidate(entry);
 	    else {
@@ -453,20 +458,25 @@ static void adjust_qos(ITF *itf,struct atm_qos *qos,int null_encap)
 }
 
 
-static void learn(VCC *vcc,uint32_t ip,struct sockaddr_atmsvc *addr)
+/*
+ * Return -1 if VCC has been closed
+ */
+
+static int learn(VCC *vcc,uint32_t ip,struct sockaddr_atmsvc *addr)
 {
     ENTRY *entry;
     ITF *itf;
-    VCC *walk;
+    VCC *walk,*next;
     unsigned char *ipp;
+    int result = 0;
 
-    if (!ip) return;
+    if (!ip) return 0;
     ipp = (unsigned char *) &ip;
     itf = lookup_itf_by_ip(ip);
     if (!itf) {
 	diag(COMPONENT,DIAG_ERROR,"got unroutable IP address %d.%d.%d.%d",
 	  ipp[0],ipp[1],ipp[2],ipp[3]);
-	return;
+	return 0;
     }
     entry = lookup_ip(itf,ip);
     assert(!vcc || vcc->entry);
@@ -477,7 +487,7 @@ static void learn(VCC *vcc,uint32_t ip,struct sockaddr_atmsvc *addr)
     if (entry && vcc && vcc->entry->itf && entry != vcc->entry) {
 	diag(COMPONENT,DIAG_DEBUG,"collision on %d.%d.%d.%d",ipp[0],ipp[1],
 	  ipp[2],ipp[3]);
-	return;
+	return 0;
     }
     /*
      * If the entry on which we received the update is dangling and we found
@@ -487,7 +497,7 @@ static void learn(VCC *vcc,uint32_t ip,struct sockaddr_atmsvc *addr)
 	if (!entry->svc) {
 	    diag(COMPONENT,DIAG_ERROR,"attempt to overwrite PVC for IP "
 	      "%d.%d.%d.%d",ipp[0],ipp[1],ipp[2],ipp[3]);
-	    return;
+	    return 0;
 	}
 	STOP_TIMER(vcc->entry);
 	Q_REMOVE(unknown_incoming,vcc->entry);
@@ -497,8 +507,12 @@ static void learn(VCC *vcc,uint32_t ip,struct sockaddr_atmsvc *addr)
 	set_sndbuf(vcc);
 	entry->flags &= ~ATF_NOVC;
 	assert(!vcc->connecting);
-	if (set_ip(vcc->fd,ip) < 0)
+	if (set_ip(vcc->fd,ip) < 0) {
 	    diag(COMPONENT,DIAG_ERROR,"set_ip: %s",strerror(errno));
+	    disconnect_vcc(vcc);
+	    vcc = NULL;
+	    result = -1;
+	}
     }
     /*
      * If we still don't have an entry, we try to use the entry that already
@@ -517,7 +531,7 @@ static void learn(VCC *vcc,uint32_t ip,struct sockaddr_atmsvc *addr)
 		    diag(COMPONENT,DIAG_ERROR,"ignoring attempt to change IP "
 		      "address of permanent entry (to %d.%d.%d.%d)",ipp[0],
 		      ipp[1],ipp[2],ipp[3]);
-		    return;
+		    return result;
 		}
 	}
 	else {
@@ -531,11 +545,11 @@ static void learn(VCC *vcc,uint32_t ip,struct sockaddr_atmsvc *addr)
       {
 	diag(COMPONENT,DIAG_ERROR,"ignoring attempt to change ATM address of "
 	  "permanent entry");
-	return;
+	return result;
     }
     if (entry->state == as_valid && entry->ip == ip && (!addr || (entry->addr
       && atm_equal((struct sockaddr *) entry->addr,(struct sockaddr *) addr,0,
-      0)))) return; /* no news */
+      0)))) return result; /* no news */
     STOP_TIMER(entry);
     if (entry->ip != ip) send_notifications(entry,0);
     entry->ip = ip;
@@ -569,10 +583,15 @@ static void learn(VCC *vcc,uint32_t ip,struct sockaddr_atmsvc *addr)
 	    }
 	}
     }
-    for (walk = entry->vccs; walk; walk = walk->next)
+    for (walk = entry->vccs; walk; walk = next) {
+	next = walk->next;
 	if (!walk->connecting)
-	    if (set_ip(walk->fd,ip) < 0)
+	    if (set_ip(walk->fd,ip) < 0) {
 		diag(COMPONENT,DIAG_ERROR,"set_ip: %s",strerror(errno));
+		disconnect_vcc(walk);
+		if (walk == vcc) result = -1;
+	    }
+    }
     if (entry->state != as_valid) {
 	if (!entry->vccs && itf->arp_srv && !(entry->flags & ATF_NOVC))
 	    connect_me(entry);
@@ -583,6 +602,7 @@ static void learn(VCC *vcc,uint32_t ip,struct sockaddr_atmsvc *addr)
 	else START_TIMER(entry,SREVAL);
     }
     entry->state = as_valid;
+    return result;
 }
 
 
@@ -797,13 +817,21 @@ void incoming_arp(VCC *vcc,struct atmarphdr *hdr,int len)
     switch (ntohs(hdr->ar_op)) {
 	case ARPOP_REQUEST:
 	    diag(COMPONENT,DIAG_DEBUG,"got ARP_REQ");
-	    learn(vcc,src_ip,&source);
+	    if (learn(vcc,src_ip,&source)) break;
 	    entry = NULL;
 	    itf = lookup_itf_by_ip(tgt_ip);
 	    entry = itf ? lookup_ip(itf,tgt_ip) : NULL;
 	    if (entry && entry->state == as_valid && (entry->flags & ATF_PUBL))
-		arp_reply(vcc,tgt_ip,entry->addr,src_ip,&source);
-	    else arp_nak(vcc,tgt_ip,src_ip,&source);
+	      {
+		if (entry->addr)
+		    arp_reply(vcc,tgt_ip,entry->addr,src_ip,&source);
+		else arp_nak(vcc,tgt_ip,src_ip,&source);
+	    }
+	    else {
+		if (itf && itf->local_ip == tgt_ip)
+		    arp_reply(vcc,tgt_ip,NULL,src_ip,&source);
+		else arp_nak(vcc,tgt_ip,src_ip,&source);
+	    }
 	    break;
 	case ARPOP_REPLY:
 	    diag(COMPONENT,DIAG_DEBUG,"got ARP_REP");
@@ -811,16 +839,15 @@ void incoming_arp(VCC *vcc,struct atmarphdr *hdr,int len)
 		diag(COMPONENT,DIAG_ERROR,"got ARP response from charlatan");
 		break;
 	    }
-	    learn(NULL,src_ip,&source);
+	    (void) learn(NULL,src_ip,&source);
 	    break;
 	case ARPOP_InREQUEST:
 	    diag(COMPONENT,DIAG_DEBUG,"got InARP_REQ");
-	    learn(vcc,src_ip,&source);
-	    inarp_reply(vcc,src_ip,&source);
+	    if (!learn(vcc,src_ip,&source)) inarp_reply(vcc,src_ip,&source);
 	    break;
 	case ARPOP_InREPLY:
 	    diag(COMPONENT,DIAG_DEBUG,"got InARP_REP");
-	    learn(vcc,src_ip,&source);
+	    (void) learn(vcc,src_ip,&source);
 	    break;
 	case ARPOP_NAK:
 	    diag(COMPONENT,DIAG_DEBUG,"got ARP_NAK");
@@ -847,7 +874,10 @@ static int ioctl_set_pvc(ITF *itf,uint32_t ip,struct sockaddr_atmpvc *addr,
     if (lookup_ip(itf,ip)) return -EEXIST; 
     if ((fd = connect_vcc((struct sockaddr *) addr,qos,sndbuf,0)) < 0)
 	return fd;
-    if ((result = set_ip(fd,ip)) < 0) return result;
+    if ((result = set_ip(fd,ip)) < 0) {
+	do_close(fd);
+	return result;
+    }
     if (flags & ATF_NULL) {
 	if ((result = set_encap(fd,0)) < 0) return result;
 	flags |= ATF_PERM;
@@ -1032,8 +1062,10 @@ void vcc_connected(VCC *vcc)
 	    }
     inarp_request(vcc->entry);
     if (vcc->entry->state == as_valid)
-	if (set_ip(vcc->fd,vcc->entry->ip) < 0)
+	if (set_ip(vcc->fd,vcc->entry->ip) < 0) {
 	    diag(COMPONENT,DIAG_ERROR,"can't set IP (%s)",strerror(errno));
+	    disconnect_vcc(vcc);
+	}
 }
 
 
