@@ -1,6 +1,6 @@
 /* table.c - ATMARP table */
  
-/* Written 1995-1999 by Werner Almesberger, EPFL-LRC/ICA */
+/* Written 1995-2000 by Werner Almesberger, EPFL-LRC/ICA */
  
 
 #include <stdio.h>
@@ -242,10 +242,11 @@ int table_update(void)
     out_error = 0;
     dump_all();
     if (fclose(out_file) < 0) out_error = errno;
-    if (!out_error)
+    if (!out_error) {
 	if (rename(ATMARP_TMP_DUMP_FILE,ATMARP_DUMP_FILE) < 0)
 	    out_error = errno;
 	else table_uptodate = 1;
+    }
     unlink(ATMARP_TMP_DUMP_FILE);
     return out_error;
 }
