@@ -58,6 +58,7 @@ void usage(char *name)
 {
   fprintf(stderr, "usage: %s [-b] [-d] [-v] [-l logfile] [-x] [-q qos]\n"
     "%13s[ -i local_ip ] [ -u uni ] <interface>\n",name,"");
+  fprintf(stderr, "%6s %s -V\n","",name);
   exit(1);
 }
 
@@ -269,7 +270,7 @@ int main(int argc, char *argv[])
   set_application("ilmid");
   set_verbosity(NULL, DIAG_WARN);
   qos = NULL;
-  while((opt = getopt(argc, argv, "bdhi:l:q:xvu:")) != EOF)
+  while((opt = getopt(argc, argv, "bdhi:l:q:xvu:V")) != EOF)
     switch (opt)
       {
       case 'd':
@@ -320,6 +321,9 @@ int main(int argc, char *argv[])
 #endif
 	}
 	break;
+      case 'V':
+	printf("%s\n",VERSION);
+	return 0;
       case 'h':
       default:
 	usage(argv[0]);
