@@ -1,6 +1,6 @@
 /* misc.c - Miscellaneous library functions */
 
-/* Written 1997,1998 by Werner Almesberger, EPFL-ICA/ICA */
+/* Written 1997-2000 by Werner Almesberger, EPFL-ICA/ICA */
 
 
 #include <stdint.h>
@@ -14,17 +14,6 @@
 #include <atmsap.h>
 
 
-int create_leaf(int session)
-{
-    int leaf;
-
-    leaf = socket(PF_ATMSVC,SOCK_DGRAM,0);
-    if (leaf < 0) return -1;
-    if (ioctl(leaf,ATM_CREATE_LEAF,session) < 0) return -1;
-    return leaf;
-}
-
-
 int __atmlib_fetch(const char **pos,...)
 {
     const char *value;
