@@ -17,6 +17,7 @@ static void usage(const char *name)
     fprintf(stderr,"usage: %s -s [ -l level ] [ -r level ] [ itf ]\n",name);
     fprintf(stderr,"%7s%s [ itf ]\n","",name);
     fprintf(stderr,"%7s%s -q [ itf ]\n","",name);
+    fprintf(stderr,"%7s%s -V\n","",name);
     fprintf(stderr,"  levels: aal = AAL PDU, atm = ATM cell,\n");
     fprintf(stderr,"%10sphy = line (digital), analog = line (analog)\n","");
     exit(1);
@@ -75,7 +76,7 @@ int main(int argc,char **argv)
     int s,c;
 
     req.number = 0;
-    while ((c = getopt(argc,argv,"l:qr:s")) != EOF)
+    while ((c = getopt(argc,argv,"l:qr:sV")) != EOF)
         switch (c) {
             case 'l':
 		if (local) usage(*argv);
@@ -91,6 +92,9 @@ int main(int argc,char **argv)
 	    case 's':
 		set = 1;
 		break;
+	    case 'V':
+		printf("%s\n",VERSION);
+		return 0;
             default:
                 usage(*argv);
         }
