@@ -662,10 +662,11 @@ static void background(void)
 
 static void usage(const char *name)
 {
-    fprintf(stderr,"%s [ -d ] [ -v ] <cmd> ...\n\n",
-      name);
+    fprintf(stderr,"usage: %s [ -d ] [ -v ] <cmd> ...\n",name);
+    fprintf(stderr,"%6s %s -V\n\n","",name);
     fprintf(stderr,"  -d  debug\n");
-    fprintf(stderr,"  -v  verbose\n\n");
+    fprintf(stderr,"  -v  verbose\n");
+    fprintf(stderr,"  -V  version\n\n");
     fprintf(stderr,"<cmd>: create [ <itf> ]  (persistent; default itf: 0)\n");
     fprintf(stderr,"       remove [ <itf> ]  (persistent; default itf: 0)\n");
     fprintf(stderr,"       virtual [ <itf> ]  (default itf: assigned by "
@@ -701,7 +702,7 @@ int main(int argc,char **argv)
     int c;
 
     FD_ZERO(&in_set);
-    while ((c = getopt(argc,argv,"dv")) != EOF)
+    while ((c = getopt(argc,argv,"dvV")) != EOF)
 	switch (c) {
 	    case 'd':
 		debug = 1;
@@ -709,6 +710,9 @@ int main(int argc,char **argv)
 	    case 'v':
 		verbose = 1;
 		break;
+	    case 'V':
+		printf("%s\n",VERSION);
+		return 0;
 	    default:
 		usage(*argv);
 	}
