@@ -305,7 +305,7 @@ int send_to_dataplane(char *buff, int length, int shortcut_fd)
 void keep_alive_sm(unsigned keep_alive_lifetime, int sequence_number){
         struct k_message msg;
 	static unsigned previous_sequence_number = 0;
-	static start_keep_alive_sm = 0;
+	static int start_keep_alive_sm = 0;
 	time_t now = time(NULL);
 	memset(&msg,0,sizeof(struct k_message));
 	if(!keep_alive_sm_running){
