@@ -317,6 +317,7 @@ static void usage(const char *name)
       "  [ -l logfile ] [ -n ] [ -m user|network|switch ]  [ -q qos ]\n"
       "  [ -t trace_length ] [ -u uni_version ] [ [itf.]vpi.vci "
       "[ socket_path ] ]\n",name);
+    fprintf(stderr,"%6s %s -V\n","",name);
     exit(1);
 }
 
@@ -334,8 +335,12 @@ int main(int argc,char **argv)
     background = 0;
     memset(&_entity.signaling_pvc,0,sizeof(_entity.signaling_pvc));
     /* 1st pass to get the -c option */
-    while ((c = getopt(argc,argv,"Abc:dD:l:m:nNP:q:t:u:")) != EOF)
+    while ((c = getopt(argc,argv,"Abc:dD:l:m:nNP:q:t:u:V")) != EOF)
 	if (c == 'c') config_file = optarg;
+	else if (c == 'V') {
+		printf("%s\n",VERSION);
+		return 0;
+	    }
     if (!(yyin = fopen(config_file,"r")))
 	diag(COMPONENT,DIAG_WARN,"%s not found. - Using defaults.",config_file);
     else if (yyparse())
