@@ -75,7 +75,7 @@ typedef struct _socket {
     SIG_ENTITY *sig;
     struct sockaddr_atmpvc pvc;
     /* --- socket layer information ---------------------------------------- */
-    atm_kptr_int_t id;
+    atm_kptr_t id;
     struct sockaddr_atmsvc local; /* local address */
     struct sockaddr_atmsvc remote; /* remote address */
     struct atm_sap sap; /* SAP (BHLI and BLLI) */
@@ -104,6 +104,8 @@ extern SOCKET *sockets;
  * stable enough to be useful to test the "real" thing against it.
  */
 
+extern atm_kptr_t kptr_null;
+
 #define S_PVC(e) \
   (e)->signaling_pvc.sap_addr.itf, \
   (e)->signaling_pvc.sap_addr.vpi, \
@@ -128,7 +130,7 @@ extern int stop;
 
 
 #define SEND_ERROR(vcc,code) \
-  send_kernel(vcc,0L,as_error,code,NULL,NULL,NULL,NULL,NULL)
+  send_kernel(vcc,kptr_null,as_error,code,NULL,NULL,NULL,NULL,NULL)
 
 
 void poll_signals(void);
@@ -137,7 +139,7 @@ void from_kernel(struct atmsvc_msg *msg,int size);
 void sync_addr(VPCI *vpci);
 
 void to_uni(SIG_ENTITY *sig,void *msg,int size);
-void send_kernel(atm_kptr_int_t vcc,atm_kptr_int_t listen_vcc,
+void send_kernel(atm_kptr_t vcc,atm_kptr_t listen_vcc,
   enum atmsvc_msg_type type,int reply,const struct sockaddr_atmpvc *pvc,
   const struct sockaddr_atmsvc *svc,const struct sockaddr_atmsvc *local,
   const struct atm_sap *sap,const struct atm_qos *qos);
@@ -148,7 +150,7 @@ void saal_okay(SIG_ENTITY *sig);
 void clear_all_calls(SIG_ENTITY *sig);
 void clear_all_calls_on_T309(SIG_ENTITY *sig);
 
-SOCKET *new_sock(atm_kptr_int_t id);
+SOCKET *new_sock(atm_kptr_t id);
 void free_sock(SOCKET *sock);
 void new_state(SOCKET *sock,STATE state);
 SOCKET *lookup_sap(const struct sockaddr_atmsvc *addr,
