@@ -97,7 +97,7 @@ char *qos_spec = NULL;
 int udp = 0;			/* 0 = tcp, !0 = udp */
 int options = 0;		/* socket options */
 int one = 1;                    /* for 4.3 BSD style setsockopt() */
-short port = 5013;		/* TCP port number */
+unsigned short port = 5013;	/* TCP port number */
 char *host;			/* ptr to name of host */
 int trans;			/* 0=receive, !0=transmit mode */
 int sinkmode = 0;		/* 0=normal I/O, !0=sink/source mode */
